==> log-front-6.1-sources.jar: io/jstuff/log/DynamicLoggerFactory.java <==
/*
 * @(#) DynamicLoggerFactory.java
 *
 * log-front  Logging interface
 * Copyright (c) 2020, 2021, 2022, 2024 Peter Wall
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package io.jstuff.log;

import java.time.Clock;

/**
 * The dynamic {@link LoggerFactory}, which tries to create a {@link Logger} as follows:
 * <ol>
 *   <li>If <code>slf4j</code> is available on the class path, create an {@link Slf4jLogger}</li>
 *   <li>If the system property <code>java.util.logging.config.file</code> is present, or if the file
 *     <code>logging.properties</code> is present as a resource, create a {@link JavaLogger}</li>
 *   <li>Otherwise, create a {@link ConsoleLogger}</li>
 * </ol>
 *
 * @author  Peter Wall
 */
public class DynamicLoggerFactory extends AbstractLoggerFactory<Logger> {

    private static Slf4jProxyInterface slf4jProxy = null;
    private static boolean javaLogging = false;

    private FormattingLoggerFactory<?, ?> formattingLoggerFactory = null;

    static {
        try {
            // first, check whether Gradle logging is present on the classpath
            slf4jProxy = new GradleProxy();
        }
        catch (Exception ignore) {
            try {
                // next, check whether slf4j is present on the classpath
                slf4jProxy = new Slf4jProxy();
            }
            catch (Exception ignore2) {
                // slf4j not found; are we using java.util.logging?
                javaLogging = System.getProperty("java.util.logging.config.file") != null ||
                        DynamicLoggerFactory.class.getResource("logging.properties") != null;
            }
        }
    }

    /**
     * Construct a {@code DefaultLoggerFactory} with the supplied default {@link Level} and {@link Clock}.
     *
     * @param   defaultLevel    the default {@link Level}
     * @param   defaultClock    the default {@link Clock}
     */
    public DynamicLoggerFactory(Level defaultLevel, Clock defaultClock) {
        super(defaultLevel, defaultClock);
    }

    /**
     * Construct a {@code DefaultLoggerFactory} with the supplied default {@link Level} and the system default
     * {@link Clock}.
     *
     * @param   defaultLevel    the default {@link Level}
     */
    public DynamicLoggerFactory(Level defaultLevel) {
        super(defaultLevel, systemClock);
    }

    /**
     * Construct a {@code DefaultLoggerFactory} with the system default {@link Level} and the supplied default
     * {@link Clock}.
     *
     * @param   defaultClock    the default {@link Clock}
     */
    public DynamicLoggerFactory(Clock defaultClock) {
        super(systemDefaultLevel, defaultClock);
    }

    /**
     * Construct a {@code DefaultLoggerFactory} with the system default {@link Level} and {@link Clock}.
     */
    public DynamicLoggerFactory() {
        super(systemDefaultLevel, systemClock);
    }

    /**
     * Get a {@link Logger} with the specified name, {@link Level} and {@link Clock}.  See the class documentation for
     * the rules used to determine the type of {@link Logger} returned.
     *
     * @param   name    the name
     * @param   level   the {@link Level}
     * @param   clock   the {@link Clock}
     * @return          a {@link Logger}
     * @throws  LoggerException     if the name is null, empty or contains non-ASCII characters
     */
    @Override
    public Logger getLogger(String name, Level level, Clock clock) {
        Logger logger = getCachedLogger(name);
        if (logger != null)
            return logger;
        logger = getSpecificLogger(name, level, clock);
        putCachedLogger(name, logger);
        return logger;
    }

    private Logger getSpecificLogger(String name, Level level, Clock clock) {
        if (slf4jProxy != null) {
            try {
                return new Slf4jLogger(name, level, clock, slf4jProxy);
            }
            catch (Exception ignore) {
            }
        }
        if (javaLogging)
            return new JavaLogger(name, level, clock);
        return getFormattingLoggerFactory().getLogger(name, level, clock);
    }

    private synchronized FormattingLoggerFactory<?, ?> getFormattingLoggerFactory() {
        if (formattingLoggerFactory == null)
            formattingLoggerFactory = FormattingLoggerFactory.getBasic();
        return formattingLoggerFactory;
    }

}

==> log-front-6.2-sources.jar: io/jstuff/log/DynamicLoggerFactory.java <==
/*
 * @(#) DynamicLoggerFactory.java
 *
 * log-front  Logging interface
 * Copyright (c) 2020, 2021, 2022, 2024, 2025 Peter Wall
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package io.jstuff.log;

import java.time.Clock;

/**
 * The dynamic {@link LoggerFactory}, which tries to create a {@link Logger} as follows:
 * <ol>
 *   <li>If <code>slf4j</code> is available on the class path, create an {@link Slf4jLogger}</li>
 *   <li>If the system property <code>java.util.logging.config.file</code> is present, or if the file
 *     <code>logging.properties</code> is present as a resource, create a {@link JavaLogger}</li>
 *   <li>Otherwise, create a {@link FormattingLogger}</li>
 * </ol>
 *
 * @author  Peter Wall
 */
public class DynamicLoggerFactory extends AbstractLoggerFactory<Logger> {

    private static Slf4jProxyInterface slf4jProxy = null;
    private static boolean javaLogging = false;

    private FormattingLoggerFactory<?, ?> formattingLoggerFactory = null;

    static {
        try {
            // first, check whether Gradle logging is present on the classpath
            slf4jProxy = new GradleProxy();
        }
        catch (Exception ignore) {
            try {
                // next, check whether slf4j is present on the classpath
                slf4jProxy = new Slf4jProxy();
            }
            catch (Exception ignore2) {
                // slf4j not found; are we using java.util.logging?
                javaLogging = System.getProperty("java.util.logging.config.file") != null ||
                        DynamicLoggerFactory.class.getResource("logging.properties") != null;
            }
        }
    }

    /**
     * Construct a {@code DefaultLoggerFactory} with the supplied default {@link Level} and {@link Clock}.
     *
     * @param   defaultLevel    the default {@link Level}
     * @param   defaultClock    the default {@link Clock}
     */
    public DynamicLoggerFactory(Level defaultLevel, Clock defaultClock) {
        super(defaultLevel, defaultClock);
    }

    /**
     * Construct a {@code DefaultLoggerFactory} with the supplied default {@link Level} and the system default
     * {@link Clock}.
     *
     * @param   defaultLevel    the default {@link Level}
     */
    public DynamicLoggerFactory(Level defaultLevel) {
        super(defaultLevel, systemClock);
    }

    /**
     * Construct a {@code DefaultLoggerFactory} with the system default {@link Level} and the supplied default
     * {@link Clock}.
     *
     * @param   defaultClock    the default {@link Clock}
     */
    public DynamicLoggerFactory(Clock defaultClock) {
        super(systemDefaultLevel, defaultClock);
    }

    /**
     * Construct a {@code DefaultLoggerFactory} with the system default {@link Level} and {@link Clock}.
     */
    public DynamicLoggerFactory() {
        super(systemDefaultLevel, systemClock);
    }

    /**
     * Create a {@link Logger} with the supplied name, {@link Level} and {@link Clock}.
     *
     * @param   name    the name
     * @param   level   the {@link Level}
     * @param   clock   the {@link Clock}
     * @return          a {@link Logger}
     */
    @Override
    protected Logger createLogger(String name, Level level, Clock clock) {
        if (slf4jProxy != null) {
            try {
                return new Slf4jLogger(name, level, clock, slf4jProxy);
            }
            catch (Exception ignore) {
            }
        }
        if (javaLogging)
            return new JavaLogger(name, level, clock);
        return getFormattingLoggerFactory().getLogger(name, level, clock);
    }

    private synchronized FormattingLoggerFactory<?, ?> getFormattingLoggerFactory() {
        if (formattingLoggerFactory == null)
            formattingLoggerFactory = FormattingLoggerFactory.getBasic();
        return formattingLoggerFactory;
    }

}
